/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package fake

import (
	"context"

	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	labels "k8s.io/apimachinery/pkg/labels"
	schema "k8s.io/apimachinery/pkg/runtime/schema"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	v1alpha1 "k8s.io/autoscaler/balancer/pkg/apis/balancer.x-k8s.io/v1alpha1"
	testing "k8s.io/client-go/testing"
)

// FakeBalancers implements BalancerInterface
type FakeBalancers struct {
	Fake *FakeBalancerV1alpha1
	ns   string
}

var balancersResource = schema.GroupVersionResource{Group: "balancer.x-k8s.io", Version: "v1alpha1", Resource: "balancers"}

var balancersKind = schema.GroupVersionKind{Group: "balancer.x-k8s.io", Version: "v1alpha1", Kind: "Balancer"}

// Get takes name of the balancer, and returns the corresponding balancer object, and an error if there is any.
func (c *FakeBalancers) Get(ctx context.Context, name string, options v1.GetOptions) (result *v1alpha1.Balancer, err error) {
	obj, err := c.Fake.
		Invokes(testing.NewGetAction(balancersResource, c.ns, name), &v1alpha1.Balancer{})

	if obj == nil {
		return nil, err
	}
	return obj.(*v1alpha1.Balancer), err
}

// List takes label and field selectors, and returns the list of Balancers that match those selectors.
func (c *FakeBalancers) List(ctx context.Context, opts v1.ListOptions) (result *v1alpha1.BalancerList, err error) {
	obj, err := c.Fake.
		Invokes(testing.NewListAction(balancersResource, balancersKind, c.ns, opts), &v1alpha1.BalancerList{})

	if obj == nil {
		return nil, err
	}

	label, _, _ := testing.ExtractFromListOptions(opts)
	if label == nil {
		label = labels.Everything()
	}
	list := &v1alpha1.BalancerList{ListMeta: obj.(*v1alpha1.BalancerList).ListMeta}
	for _, item := range obj.(*v1alpha1.BalancerList).Items {
		if label.Matches(labels.Set(item.Labels)) {
			list.Items = append(list.Items, item)
		}
	}
	return list, err
}

// Watch returns a watch.Interface that watches the requested balancers.
func (c *FakeBalancers) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	return c.Fake.
		InvokesWatch(testing.NewWatchAction(balancersResource, c.ns, opts))

}

// Create takes the representation of a balancer and creates it.  Returns the server's representation of the balancer, and an error, if there is any.
func (c *FakeBalancers) Create(ctx context.Context, balancer *v1alpha1.Balancer, opts v1.CreateOptions) (result *v1alpha1.Balancer, err error) {
	obj, err := c.Fake.
		Invokes(testing.NewCreateAction(balancersResource, c.ns, balancer), &v1alpha1.Balancer{})

	if obj == nil {
		return nil, err
	}
	return obj.(*v1alpha1.Balancer), err
}

// Update takes the representation of a balancer and updates it. Returns the server's representation of the balancer, and an error, if there is any.
func (c *FakeBalancers) Update(ctx context.Context, balancer *v1alpha1.Balancer, opts v1.UpdateOptions) (result *v1alpha1.Balancer, err error) {
	obj, err := c.Fake.
		Invokes(testing.NewUpdateAction(balancersResource, c.ns, balancer), &v1alpha1.Balancer{})

	if obj == nil {
		return nil, err
	}
	return obj.(*v1alpha1.Balancer), err
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *FakeBalancers) UpdateStatus(ctx context.Context, balancer *v1alpha1.Balancer, opts v1.UpdateOptions) (*v1alpha1.Balancer, error) {
	obj, err := c.Fake.
		Invokes(testing.NewUpdateSubresourceAction(balancersResource, "status", c.ns, balancer), &v1alpha1.Balancer{})

	if obj == nil {
		return nil, err
	}
	return obj.(*v1alpha1.Balancer), err
}

// Delete takes name of the balancer and deletes it. Returns an error if one occurs.
func (c *FakeBalancers) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	_, err := c.Fake.
		Invokes(testing.NewDeleteActionWithOptions(balancersResource, c.ns, name, opts), &v1alpha1.Balancer{})

	return err
}

// DeleteCollection deletes a collection of objects.
func (c *FakeBalancers) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	action := testing.NewDeleteCollectionAction(balancersResource, c.ns, listOpts)

	_, err := c.Fake.Invokes(action, &v1alpha1.BalancerList{})
	return err
}

// Patch applies the patch and returns the patched balancer.
func (c *FakeBalancers) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.Balancer, err error) {
	obj, err := c.Fake.
		Invokes(testing.NewPatchSubresourceAction(balancersResource, c.ns, name, pt, data, subresources...), &v1alpha1.Balancer{})

	if obj == nil {
		return nil, err
	}
	return obj.(*v1alpha1.Balancer), err
}
