/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package v1beta1

import (
	"context"
	json "encoding/json"
	"fmt"
	"time"

	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	v1beta1 "k8s.io/autoscaler/cluster-autoscaler/apis/provisioningrequest/autoscaling.x-k8s.io/v1beta1"
	autoscalingxk8siov1beta1 "k8s.io/autoscaler/cluster-autoscaler/apis/provisioningrequest/client/applyconfiguration/autoscaling.x-k8s.io/v1beta1"
	scheme "k8s.io/autoscaler/cluster-autoscaler/apis/provisioningrequest/client/clientset/versioned/scheme"
	rest "k8s.io/client-go/rest"
)

// ProvisioningRequestsGetter has a method to return a ProvisioningRequestInterface.
// A group's client should implement this interface.
type ProvisioningRequestsGetter interface {
	ProvisioningRequests(namespace string) ProvisioningRequestInterface
}

// ProvisioningRequestInterface has methods to work with ProvisioningRequest resources.
type ProvisioningRequestInterface interface {
	Create(ctx context.Context, provisioningRequest *v1beta1.ProvisioningRequest, opts v1.CreateOptions) (*v1beta1.ProvisioningRequest, error)
	Update(ctx context.Context, provisioningRequest *v1beta1.ProvisioningRequest, opts v1.UpdateOptions) (*v1beta1.ProvisioningRequest, error)
	UpdateStatus(ctx context.Context, provisioningRequest *v1beta1.ProvisioningRequest, opts v1.UpdateOptions) (*v1beta1.ProvisioningRequest, error)
	Delete(ctx context.Context, name string, opts v1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error
	Get(ctx context.Context, name string, opts v1.GetOptions) (*v1beta1.ProvisioningRequest, error)
	List(ctx context.Context, opts v1.ListOptions) (*v1beta1.ProvisioningRequestList, error)
	Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1beta1.ProvisioningRequest, err error)
	Apply(ctx context.Context, provisioningRequest *autoscalingxk8siov1beta1.ProvisioningRequestApplyConfiguration, opts v1.ApplyOptions) (result *v1beta1.ProvisioningRequest, err error)
	ApplyStatus(ctx context.Context, provisioningRequest *autoscalingxk8siov1beta1.ProvisioningRequestApplyConfiguration, opts v1.ApplyOptions) (result *v1beta1.ProvisioningRequest, err error)
	ProvisioningRequestExpansion
}

// provisioningRequests implements ProvisioningRequestInterface
type provisioningRequests struct {
	client rest.Interface
	ns     string
}

// newProvisioningRequests returns a ProvisioningRequests
func newProvisioningRequests(c *AutoscalingV1beta1Client, namespace string) *provisioningRequests {
	return &provisioningRequests{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the provisioningRequest, and returns the corresponding provisioningRequest object, and an error if there is any.
func (c *provisioningRequests) Get(ctx context.Context, name string, options v1.GetOptions) (result *v1beta1.ProvisioningRequest, err error) {
	result = &v1beta1.ProvisioningRequest{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("provisioningrequests").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of ProvisioningRequests that match those selectors.
func (c *provisioningRequests) List(ctx context.Context, opts v1.ListOptions) (result *v1beta1.ProvisioningRequestList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1beta1.ProvisioningRequestList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("provisioningrequests").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested provisioningRequests.
func (c *provisioningRequests) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("provisioningrequests").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a provisioningRequest and creates it.  Returns the server's representation of the provisioningRequest, and an error, if there is any.
func (c *provisioningRequests) Create(ctx context.Context, provisioningRequest *v1beta1.ProvisioningRequest, opts v1.CreateOptions) (result *v1beta1.ProvisioningRequest, err error) {
	result = &v1beta1.ProvisioningRequest{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("provisioningrequests").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(provisioningRequest).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a provisioningRequest and updates it. Returns the server's representation of the provisioningRequest, and an error, if there is any.
func (c *provisioningRequests) Update(ctx context.Context, provisioningRequest *v1beta1.ProvisioningRequest, opts v1.UpdateOptions) (result *v1beta1.ProvisioningRequest, err error) {
	result = &v1beta1.ProvisioningRequest{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("provisioningrequests").
		Name(provisioningRequest.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(provisioningRequest).
		Do(ctx).
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *provisioningRequests) UpdateStatus(ctx context.Context, provisioningRequest *v1beta1.ProvisioningRequest, opts v1.UpdateOptions) (result *v1beta1.ProvisioningRequest, err error) {
	result = &v1beta1.ProvisioningRequest{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("provisioningrequests").
		Name(provisioningRequest.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(provisioningRequest).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the provisioningRequest and deletes it. Returns an error if one occurs.
func (c *provisioningRequests) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("provisioningrequests").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *provisioningRequests) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("provisioningrequests").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched provisioningRequest.
func (c *provisioningRequests) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1beta1.ProvisioningRequest, err error) {
	result = &v1beta1.ProvisioningRequest{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("provisioningrequests").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}

// Apply takes the given apply declarative configuration, applies it and returns the applied provisioningRequest.
func (c *provisioningRequests) Apply(ctx context.Context, provisioningRequest *autoscalingxk8siov1beta1.ProvisioningRequestApplyConfiguration, opts v1.ApplyOptions) (result *v1beta1.ProvisioningRequest, err error) {
	if provisioningRequest == nil {
		return nil, fmt.Errorf("provisioningRequest provided to Apply must not be nil")
	}
	patchOpts := opts.ToPatchOptions()
	data, err := json.Marshal(provisioningRequest)
	if err != nil {
		return nil, err
	}
	name := provisioningRequest.Name
	if name == nil {
		return nil, fmt.Errorf("provisioningRequest.Name must be provided to Apply")
	}
	result = &v1beta1.ProvisioningRequest{}
	err = c.client.Patch(types.ApplyPatchType).
		Namespace(c.ns).
		Resource("provisioningrequests").
		Name(*name).
		VersionedParams(&patchOpts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}

// ApplyStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating ApplyStatus().
func (c *provisioningRequests) ApplyStatus(ctx context.Context, provisioningRequest *autoscalingxk8siov1beta1.ProvisioningRequestApplyConfiguration, opts v1.ApplyOptions) (result *v1beta1.ProvisioningRequest, err error) {
	if provisioningRequest == nil {
		return nil, fmt.Errorf("provisioningRequest provided to Apply must not be nil")
	}
	patchOpts := opts.ToPatchOptions()
	data, err := json.Marshal(provisioningRequest)
	if err != nil {
		return nil, err
	}

	name := provisioningRequest.Name
	if name == nil {
		return nil, fmt.Errorf("provisioningRequest.Name must be provided to Apply")
	}

	result = &v1beta1.ProvisioningRequest{}
	err = c.client.Patch(types.ApplyPatchType).
		Namespace(c.ns).
		Resource("provisioningrequests").
		Name(*name).
		SubResource("status").
		VersionedParams(&patchOpts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
